# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.insights.v1.call.annotation import AnnotationList
from twilio.rest.insights.v1.call.event import EventList
from twilio.rest.insights.v1.call.metric import MetricList
from twilio.rest.insights.v1.call.summary import CallSummaryList


class CallList(ListResource):

    def __init__(self, version):
        """
        Initialize the CallList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.insights.v1.call.CallList
        :rtype: twilio.rest.insights.v1.call.CallList
        """
        super(CallList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, sid):
        """
        Constructs a CallContext

        :param sid: The sid

        :returns: twilio.rest.insights.v1.call.CallContext
        :rtype: twilio.rest.insights.v1.call.CallContext
        """
        return CallContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a CallContext

        :param sid: The sid

        :returns: twilio.rest.insights.v1.call.CallContext
        :rtype: twilio.rest.insights.v1.call.CallContext
        """
        return CallContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallList>'


class CallPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the CallPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.insights.v1.call.CallPage
        :rtype: twilio.rest.insights.v1.call.CallPage
        """
        super(CallPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of CallInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.call.CallInstance
        :rtype: twilio.rest.insights.v1.call.CallInstance
        """
        return CallInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallPage>'


class CallContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the CallContext

        :param Version version: Version that contains the resource
        :param sid: The sid

        :returns: twilio.rest.insights.v1.call.CallContext
        :rtype: twilio.rest.insights.v1.call.CallContext
        """
        super(CallContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Voice/{sid}'.format(**self._solution)

        # Dependents
        self._events = None
        self._metrics = None
        self._summary = None
        self._annotation = None

    def fetch(self):
        """
        Fetch the CallInstance

        :returns: The fetched CallInstance
        :rtype: twilio.rest.insights.v1.call.CallInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return CallInstance(self._version, payload, sid=self._solution['sid'], )

    @property
    def events(self):
        """
        Access the events

        :returns: twilio.rest.insights.v1.call.event.EventList
        :rtype: twilio.rest.insights.v1.call.event.EventList
        """
        if self._events is None:
            self._events = EventList(self._version, call_sid=self._solution['sid'], )
        return self._events

    @property
    def metrics(self):
        """
        Access the metrics

        :returns: twilio.rest.insights.v1.call.metric.MetricList
        :rtype: twilio.rest.insights.v1.call.metric.MetricList
        """
        if self._metrics is None:
            self._metrics = MetricList(self._version, call_sid=self._solution['sid'], )
        return self._metrics

    @property
    def summary(self):
        """
        Access the summary

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryList
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryList
        """
        if self._summary is None:
            self._summary = CallSummaryList(self._version, call_sid=self._solution['sid'], )
        return self._summary

    @property
    def annotation(self):
        """
        Access the annotation

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationList
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationList
        """
        if self._annotation is None:
            self._annotation = AnnotationList(self._version, call_sid=self._solution['sid'], )
        return self._annotation

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.CallContext {}>'.format(context)


class CallInstance(InstanceResource):

    def __init__(self, version, payload, sid=None):
        """
        Initialize the CallInstance

        :returns: twilio.rest.insights.v1.call.CallInstance
        :rtype: twilio.rest.insights.v1.call.CallInstance
        """
        super(CallInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: CallContext for this CallInstance
        :rtype: twilio.rest.insights.v1.call.CallContext
        """
        if self._context is None:
            self._context = CallContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The sid
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The links
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the CallInstance

        :returns: The fetched CallInstance
        :rtype: twilio.rest.insights.v1.call.CallInstance
        """
        return self._proxy.fetch()

    @property
    def events(self):
        """
        Access the events

        :returns: twilio.rest.insights.v1.call.event.EventList
        :rtype: twilio.rest.insights.v1.call.event.EventList
        """
        return self._proxy.events

    @property
    def metrics(self):
        """
        Access the metrics

        :returns: twilio.rest.insights.v1.call.metric.MetricList
        :rtype: twilio.rest.insights.v1.call.metric.MetricList
        """
        return self._proxy.metrics

    @property
    def summary(self):
        """
        Access the summary

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryList
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryList
        """
        return self._proxy.summary

    @property
    def annotation(self):
        """
        Access the annotation

        :returns: twilio.rest.insights.v1.call.annotation.AnnotationList
        :rtype: twilio.rest.insights.v1.call.annotation.AnnotationList
        """
        return self._proxy.annotation

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.CallInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.pricing.v1 import V1
from twilio.rest.pricing.v2 import V2


class Pricing(Domain):

    def __init__(self, twilio):
        """
        Initialize the Pricing Domain

        :returns: Domain for Pricing
        :rtype: twilio.rest.pricing.Pricing
        """
        super(Pricing, self).__init__(twilio)

        self.base_url = 'https://pricing.twilio.com'

        # Versions
        self._v1 = None
        self._v2 = None

    @property
    def v1(self):
        """
        :returns: Version v1 of pricing
        :rtype: twilio.rest.pricing.v1.V1
        """
        if self._v1 is None:
            self._v1 = V1(self)
        return self._v1

    @property
    def v2(self):
        """
        :returns: Version v2 of pricing
        :rtype: twilio.rest.pricing.v2.V2
        """
        if self._v2 is None:
            self._v2 = V2(self)
        return self._v2

    @property
    def messaging(self):
        """
        :rtype: twilio.rest.pricing.v1.messaging.MessagingList
        """
        return self.v1.messaging

    @property
    def phone_numbers(self):
        """
        :rtype: twilio.rest.pricing.v1.phone_number.PhoneNumberList
        """
        return self.v1.phone_numbers

    @property
    def voice(self):
        """
        :rtype: twilio.rest.pricing.v2.voice.VoiceList
        """
        return self.v2.voice

    @property
    def countries(self):
        """
        :rtype: twilio.rest.pricing.v2.country.CountryList
        """
        return self.v2.countries

    @property
    def numbers(self):
        """
        :rtype: twilio.rest.pricing.v2.number.NumberList
        """
        return self.v2.numbers

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Pricing>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.verify.v2.service.entity.challenge import ChallengeList
from twilio.rest.verify.v2.service.entity.factor import FactorList
from twilio.rest.verify.v2.service.entity.new_factor import NewFactorList


class EntityList(ListResource):

    def __init__(self, version, service_sid):
        """
        Initialize the EntityList

        :param Version version: Version that contains the resource
        :param service_sid: Service Sid.

        :returns: twilio.rest.verify.v2.service.entity.EntityList
        :rtype: twilio.rest.verify.v2.service.entity.EntityList
        """
        super(EntityList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, }
        self._uri = '/Services/{service_sid}/Entities'.format(**self._solution)

    def create(self, identity):
        """
        Create the EntityInstance

        :param unicode identity: Unique external identifier of the Entity

        :returns: The created EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityInstance
        """
        data = values.of({'Identity': identity, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return EntityInstance(self._version, payload, service_sid=self._solution['service_sid'], )

    def stream(self, limit=None, page_size=None):
        """
        Streams EntityInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.entity.EntityInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists EntityInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.entity.EntityInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of EntityInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return EntityPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of EntityInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return EntityPage(self._version, response, self._solution)

    def get(self, identity):
        """
        Constructs a EntityContext

        :param identity: Unique external identifier of the Entity

        :returns: twilio.rest.verify.v2.service.entity.EntityContext
        :rtype: twilio.rest.verify.v2.service.entity.EntityContext
        """
        return EntityContext(self._version, service_sid=self._solution['service_sid'], identity=identity, )

    def __call__(self, identity):
        """
        Constructs a EntityContext

        :param identity: Unique external identifier of the Entity

        :returns: twilio.rest.verify.v2.service.entity.EntityContext
        :rtype: twilio.rest.verify.v2.service.entity.EntityContext
        """
        return EntityContext(self._version, service_sid=self._solution['service_sid'], identity=identity, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.EntityList>'


class EntityPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the EntityPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: Service Sid.

        :returns: twilio.rest.verify.v2.service.entity.EntityPage
        :rtype: twilio.rest.verify.v2.service.entity.EntityPage
        """
        super(EntityPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of EntityInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.service.entity.EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityInstance
        """
        return EntityInstance(self._version, payload, service_sid=self._solution['service_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.EntityPage>'


class EntityContext(InstanceContext):

    def __init__(self, version, service_sid, identity):
        """
        Initialize the EntityContext

        :param Version version: Version that contains the resource
        :param service_sid: Service Sid.
        :param identity: Unique external identifier of the Entity

        :returns: twilio.rest.verify.v2.service.entity.EntityContext
        :rtype: twilio.rest.verify.v2.service.entity.EntityContext
        """
        super(EntityContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'identity': identity, }
        self._uri = '/Services/{service_sid}/Entities/{identity}'.format(**self._solution)

        # Dependents
        self._factors = None
        self._new_factors = None
        self._challenges = None

    def delete(self):
        """
        Deletes the EntityInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def fetch(self):
        """
        Fetch the EntityInstance

        :returns: The fetched EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return EntityInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
        )

    @property
    def factors(self):
        """
        Access the factors

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorList
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorList
        """
        if self._factors is None:
            self._factors = FactorList(
                self._version,
                service_sid=self._solution['service_sid'],
                identity=self._solution['identity'],
            )
        return self._factors

    @property
    def new_factors(self):
        """
        Access the new_factors

        :returns: twilio.rest.verify.v2.service.entity.new_factor.NewFactorList
        :rtype: twilio.rest.verify.v2.service.entity.new_factor.NewFactorList
        """
        if self._new_factors is None:
            self._new_factors = NewFactorList(
                self._version,
                service_sid=self._solution['service_sid'],
                identity=self._solution['identity'],
            )
        return self._new_factors

    @property
    def challenges(self):
        """
        Access the challenges

        :returns: twilio.rest.verify.v2.service.entity.challenge.ChallengeList
        :rtype: twilio.rest.verify.v2.service.entity.challenge.ChallengeList
        """
        if self._challenges is None:
            self._challenges = ChallengeList(
                self._version,
                service_sid=self._solution['service_sid'],
                identity=self._solution['identity'],
            )
        return self._challenges

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.EntityContext {}>'.format(context)


class EntityInstance(InstanceResource):

    def __init__(self, version, payload, service_sid, identity=None):
        """
        Initialize the EntityInstance

        :returns: twilio.rest.verify.v2.service.entity.EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityInstance
        """
        super(EntityInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'identity': payload.get('identity'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'service_sid': service_sid, 'identity': identity or self._properties['identity'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: EntityContext for this EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityContext
        """
        if self._context is None:
            self._context = EntityContext(
                self._version,
                service_sid=self._solution['service_sid'],
                identity=self._solution['identity'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies this Entity.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def identity(self):
        """
        :returns: Unique external identifier of the Entity
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def account_sid(self):
        """
        :returns: Account Sid.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: Service Sid.
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def date_created(self):
        """
        :returns: The date this Entity was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date this Entity was updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: Nested resource URLs.
        :rtype: unicode
        """
        return self._properties['links']

    def delete(self):
        """
        Deletes the EntityInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def fetch(self):
        """
        Fetch the EntityInstance

        :returns: The fetched EntityInstance
        :rtype: twilio.rest.verify.v2.service.entity.EntityInstance
        """
        return self._proxy.fetch()

    @property
    def factors(self):
        """
        Access the factors

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorList
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorList
        """
        return self._proxy.factors

    @property
    def new_factors(self):
        """
        Access the new_factors

        :returns: twilio.rest.verify.v2.service.entity.new_factor.NewFactorList
        :rtype: twilio.rest.verify.v2.service.entity.new_factor.NewFactorList
        """
        return self._proxy.new_factors

    @property
    def challenges(self):
        """
        Access the challenges

        :returns: twilio.rest.verify.v2.service.entity.challenge.ChallengeList
        :rtype: twilio.rest.verify.v2.service.entity.challenge.ChallengeList
        """
        return self._proxy.challenges

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.EntityInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class WorkspaceStatisticsList(ListResource):

    def __init__(self, version, workspace_sid):
        """
        Initialize the WorkspaceStatisticsList

        :param Version version: Version that contains the resource
        :param workspace_sid: The SID of the Workspace

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsList
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsList
        """
        super(WorkspaceStatisticsList, self).__init__(version)

        # Path Solution
        self._solution = {'workspace_sid': workspace_sid, }

    def get(self):
        """
        Constructs a WorkspaceStatisticsContext

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        """
        return WorkspaceStatisticsContext(self._version, workspace_sid=self._solution['workspace_sid'], )

    def __call__(self):
        """
        Constructs a WorkspaceStatisticsContext

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        """
        return WorkspaceStatisticsContext(self._version, workspace_sid=self._solution['workspace_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Taskrouter.V1.WorkspaceStatisticsList>'


class WorkspaceStatisticsPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the WorkspaceStatisticsPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param workspace_sid: The SID of the Workspace

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsPage
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsPage
        """
        super(WorkspaceStatisticsPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of WorkspaceStatisticsInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        """
        return WorkspaceStatisticsInstance(
            self._version,
            payload,
            workspace_sid=self._solution['workspace_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Taskrouter.V1.WorkspaceStatisticsPage>'


class WorkspaceStatisticsContext(InstanceContext):

    def __init__(self, version, workspace_sid):
        """
        Initialize the WorkspaceStatisticsContext

        :param Version version: Version that contains the resource
        :param workspace_sid: The SID of the Workspace to fetch

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        """
        super(WorkspaceStatisticsContext, self).__init__(version)

        # Path Solution
        self._solution = {'workspace_sid': workspace_sid, }
        self._uri = '/Workspaces/{workspace_sid}/Statistics'.format(**self._solution)

    def fetch(self, minutes=values.unset, start_date=values.unset,
              end_date=values.unset, task_channel=values.unset,
              split_by_wait_time=values.unset):
        """
        Fetch the WorkspaceStatisticsInstance

        :param unicode minutes: Only calculate statistics since this many minutes in the past
        :param datetime start_date: Only calculate statistics from on or after this date
        :param datetime end_date: Only calculate statistics from this date and time and earlier
        :param unicode task_channel: Only calculate statistics on this TaskChannel.
        :param unicode split_by_wait_time: A comma separated list of values that describes the thresholds to calculate statistics on

        :returns: The fetched WorkspaceStatisticsInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        """
        data = values.of({
            'Minutes': minutes,
            'StartDate': serialize.iso8601_datetime(start_date),
            'EndDate': serialize.iso8601_datetime(end_date),
            'TaskChannel': task_channel,
            'SplitByWaitTime': split_by_wait_time,
        })

        payload = self._version.fetch(method='GET', uri=self._uri, params=data, )

        return WorkspaceStatisticsInstance(
            self._version,
            payload,
            workspace_sid=self._solution['workspace_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Taskrouter.V1.WorkspaceStatisticsContext {}>'.format(context)


class WorkspaceStatisticsInstance(InstanceResource):

    def __init__(self, version, payload, workspace_sid):
        """
        Initialize the WorkspaceStatisticsInstance

        :returns: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        """
        super(WorkspaceStatisticsInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'realtime': payload.get('realtime'),
            'cumulative': payload.get('cumulative'),
            'account_sid': payload.get('account_sid'),
            'workspace_sid': payload.get('workspace_sid'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'workspace_sid': workspace_sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: WorkspaceStatisticsContext for this WorkspaceStatisticsInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsContext
        """
        if self._context is None:
            self._context = WorkspaceStatisticsContext(
                self._version,
                workspace_sid=self._solution['workspace_sid'],
            )
        return self._context

    @property
    def realtime(self):
        """
        :returns: n object that contains the real-time statistics for the Workspace
        :rtype: dict
        """
        return self._properties['realtime']

    @property
    def cumulative(self):
        """
        :returns: An object that contains the cumulative statistics for the Workspace
        :rtype: dict
        """
        return self._properties['cumulative']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def workspace_sid(self):
        """
        :returns: The SID of the Workspace
        :rtype: unicode
        """
        return self._properties['workspace_sid']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Workspace statistics resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self, minutes=values.unset, start_date=values.unset,
              end_date=values.unset, task_channel=values.unset,
              split_by_wait_time=values.unset):
        """
        Fetch the WorkspaceStatisticsInstance

        :param unicode minutes: Only calculate statistics since this many minutes in the past
        :param datetime start_date: Only calculate statistics from on or after this date
        :param datetime end_date: Only calculate statistics from this date and time and earlier
        :param unicode task_channel: Only calculate statistics on this TaskChannel.
        :param unicode split_by_wait_time: A comma separated list of values that describes the thresholds to calculate statistics on

        :returns: The fetched WorkspaceStatisticsInstance
        :rtype: twilio.rest.taskrouter.v1.workspace.workspace_statistics.WorkspaceStatisticsInstance
        """
        return self._proxy.fetch(
            minutes=minutes,
            start_date=start_date,
            end_date=end_date,
            task_channel=task_channel,
            split_by_wait_time=split_by_wait_time,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Taskrouter.V1.WorkspaceStatisticsInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class OpenidDiscoveryList(ListResource):

    def __init__(self, version):
        """
        Initialize the OpenidDiscoveryList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryList
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryList
        """
        super(OpenidDiscoveryList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a OpenidDiscoveryContext

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        """
        return OpenidDiscoveryContext(self._version, )

    def __call__(self):
        """
        Constructs a OpenidDiscoveryContext

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        """
        return OpenidDiscoveryContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.OpenidDiscoveryList>'


class OpenidDiscoveryPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the OpenidDiscoveryPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryPage
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryPage
        """
        super(OpenidDiscoveryPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of OpenidDiscoveryInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        """
        return OpenidDiscoveryInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.OpenidDiscoveryPage>'


class OpenidDiscoveryContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the OpenidDiscoveryContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        """
        super(OpenidDiscoveryContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/well-known/openid-configuration'.format(**self._solution)

    def fetch(self):
        """
        Fetch the OpenidDiscoveryInstance

        :returns: The fetched OpenidDiscoveryInstance
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return OpenidDiscoveryInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.OpenidDiscoveryContext {}>'.format(context)


class OpenidDiscoveryInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the OpenidDiscoveryInstance

        :returns: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        """
        super(OpenidDiscoveryInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'issuer': payload.get('issuer'),
            'authorization_endpoint': payload.get('authorization_endpoint'),
            'device_authorization_endpoint': payload.get('device_authorization_endpoint'),
            'token_endpoint': payload.get('token_endpoint'),
            'userinfo_endpoint': payload.get('userinfo_endpoint'),
            'revocation_endpoint': payload.get('revocation_endpoint'),
            'jwk_uri': payload.get('jwk_uri'),
            'response_type_supported': payload.get('response_type_supported'),
            'subject_type_supported': payload.get('subject_type_supported'),
            'id_token_signing_alg_values_supported': payload.get('id_token_signing_alg_values_supported'),
            'scopes_supported': payload.get('scopes_supported'),
            'claims_supported': payload.get('claims_supported'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: OpenidDiscoveryContext for this OpenidDiscoveryInstance
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryContext
        """
        if self._context is None:
            self._context = OpenidDiscoveryContext(self._version, )
        return self._context

    @property
    def issuer(self):
        """
        :returns: The issuer URL
        :rtype: unicode
        """
        return self._properties['issuer']

    @property
    def authorization_endpoint(self):
        """
        :returns: The URL of authorization endpoint
        :rtype: unicode
        """
        return self._properties['authorization_endpoint']

    @property
    def device_authorization_endpoint(self):
        """
        :returns: The URL of device code authorization endpoint
        :rtype: unicode
        """
        return self._properties['device_authorization_endpoint']

    @property
    def token_endpoint(self):
        """
        :returns: The URL of token endpoint
        :rtype: unicode
        """
        return self._properties['token_endpoint']

    @property
    def userinfo_endpoint(self):
        """
        :returns: The URL of user info endpoint
        :rtype: unicode
        """
        return self._properties['userinfo_endpoint']

    @property
    def revocation_endpoint(self):
        """
        :returns: The URL of revocation endpoint
        :rtype: unicode
        """
        return self._properties['revocation_endpoint']

    @property
    def jwk_uri(self):
        """
        :returns: The URL of public JWK endpoint
        :rtype: unicode
        """
        return self._properties['jwk_uri']

    @property
    def response_type_supported(self):
        """
        :returns: List of response type supported for identity token
        :rtype: list[unicode]
        """
        return self._properties['response_type_supported']

    @property
    def subject_type_supported(self):
        """
        :returns: List of subject supported for identity token
        :rtype: list[unicode]
        """
        return self._properties['subject_type_supported']

    @property
    def id_token_signing_alg_values_supported(self):
        """
        :returns: List of JWS signing algorithms supported for identity token
        :rtype: list[unicode]
        """
        return self._properties['id_token_signing_alg_values_supported']

    @property
    def scopes_supported(self):
        """
        :returns: List of scopes supported identity token
        :rtype: list[unicode]
        """
        return self._properties['scopes_supported']

    @property
    def claims_supported(self):
        """
        :returns: List of claims supported for identity token
        :rtype: list[unicode]
        """
        return self._properties['claims_supported']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the OpenidDiscoveryInstance

        :returns: The fetched OpenidDiscoveryInstance
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.OpenidDiscoveryInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class CustomerProfilesChannelEndpointAssignmentList(ListResource):

    def __init__(self, version, customer_profile_sid):
        """
        Initialize the CustomerProfilesChannelEndpointAssignmentList

        :param Version version: Version that contains the resource
        :param customer_profile_sid: The unique string that identifies the CustomerProfile resource.

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentList
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentList
        """
        super(CustomerProfilesChannelEndpointAssignmentList, self).__init__(version)

        # Path Solution
        self._solution = {'customer_profile_sid': customer_profile_sid, }
        self._uri = '/CustomerProfiles/{customer_profile_sid}/ChannelEndpointAssignments'.format(**self._solution)

    def create(self, channel_endpoint_type, channel_endpoint_sid):
        """
        Create the CustomerProfilesChannelEndpointAssignmentInstance

        :param unicode channel_endpoint_type: The type of channel endpoint
        :param unicode channel_endpoint_sid: The sid of an channel endpoint

        :returns: The created CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        """
        data = values.of({
            'ChannelEndpointType': channel_endpoint_type,
            'ChannelEndpointSid': channel_endpoint_sid,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return CustomerProfilesChannelEndpointAssignmentInstance(
            self._version,
            payload,
            customer_profile_sid=self._solution['customer_profile_sid'],
        )

    def stream(self, channel_endpoint_sid=values.unset,
               channel_endpoint_sids=values.unset, limit=None, page_size=None):
        """
        Streams CustomerProfilesChannelEndpointAssignmentInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode channel_endpoint_sid: The sid of an channel endpoint
        :param unicode channel_endpoint_sids: comma separated list of channel endpoint sids
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            channel_endpoint_sid=channel_endpoint_sid,
            channel_endpoint_sids=channel_endpoint_sids,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, channel_endpoint_sid=values.unset,
             channel_endpoint_sids=values.unset, limit=None, page_size=None):
        """
        Lists CustomerProfilesChannelEndpointAssignmentInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode channel_endpoint_sid: The sid of an channel endpoint
        :param unicode channel_endpoint_sids: comma separated list of channel endpoint sids
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance]
        """
        return list(self.stream(
            channel_endpoint_sid=channel_endpoint_sid,
            channel_endpoint_sids=channel_endpoint_sids,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, channel_endpoint_sid=values.unset,
             channel_endpoint_sids=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of CustomerProfilesChannelEndpointAssignmentInstance records from the API.
        Request is executed immediately

        :param unicode channel_endpoint_sid: The sid of an channel endpoint
        :param unicode channel_endpoint_sids: comma separated list of channel endpoint sids
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentPage
        """
        data = values.of({
            'ChannelEndpointSid': channel_endpoint_sid,
            'ChannelEndpointSids': channel_endpoint_sids,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return CustomerProfilesChannelEndpointAssignmentPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of CustomerProfilesChannelEndpointAssignmentInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return CustomerProfilesChannelEndpointAssignmentPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a CustomerProfilesChannelEndpointAssignmentContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        """
        return CustomerProfilesChannelEndpointAssignmentContext(
            self._version,
            customer_profile_sid=self._solution['customer_profile_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a CustomerProfilesChannelEndpointAssignmentContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        """
        return CustomerProfilesChannelEndpointAssignmentContext(
            self._version,
            customer_profile_sid=self._solution['customer_profile_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Trusthub.V1.CustomerProfilesChannelEndpointAssignmentList>'


class CustomerProfilesChannelEndpointAssignmentPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the CustomerProfilesChannelEndpointAssignmentPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param customer_profile_sid: The unique string that identifies the CustomerProfile resource.

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentPage
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentPage
        """
        super(CustomerProfilesChannelEndpointAssignmentPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of CustomerProfilesChannelEndpointAssignmentInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        """
        return CustomerProfilesChannelEndpointAssignmentInstance(
            self._version,
            payload,
            customer_profile_sid=self._solution['customer_profile_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Trusthub.V1.CustomerProfilesChannelEndpointAssignmentPage>'


class CustomerProfilesChannelEndpointAssignmentContext(InstanceContext):

    def __init__(self, version, customer_profile_sid, sid):
        """
        Initialize the CustomerProfilesChannelEndpointAssignmentContext

        :param Version version: Version that contains the resource
        :param customer_profile_sid: The unique string that identifies the resource.
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        """
        super(CustomerProfilesChannelEndpointAssignmentContext, self).__init__(version)

        # Path Solution
        self._solution = {'customer_profile_sid': customer_profile_sid, 'sid': sid, }
        self._uri = '/CustomerProfiles/{customer_profile_sid}/ChannelEndpointAssignments/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the CustomerProfilesChannelEndpointAssignmentInstance

        :returns: The fetched CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return CustomerProfilesChannelEndpointAssignmentInstance(
            self._version,
            payload,
            customer_profile_sid=self._solution['customer_profile_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the CustomerProfilesChannelEndpointAssignmentInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Trusthub.V1.CustomerProfilesChannelEndpointAssignmentContext {}>'.format(context)


class CustomerProfilesChannelEndpointAssignmentInstance(InstanceResource):

    def __init__(self, version, payload, customer_profile_sid, sid=None):
        """
        Initialize the CustomerProfilesChannelEndpointAssignmentInstance

        :returns: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        """
        super(CustomerProfilesChannelEndpointAssignmentInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'customer_profile_sid': payload.get('customer_profile_sid'),
            'account_sid': payload.get('account_sid'),
            'channel_endpoint_type': payload.get('channel_endpoint_type'),
            'channel_endpoint_sid': payload.get('channel_endpoint_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'customer_profile_sid': customer_profile_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: CustomerProfilesChannelEndpointAssignmentContext for this CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentContext
        """
        if self._context is None:
            self._context = CustomerProfilesChannelEndpointAssignmentContext(
                self._version,
                customer_profile_sid=self._solution['customer_profile_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def customer_profile_sid(self):
        """
        :returns: The unique string that identifies the CustomerProfile resource.
        :rtype: unicode
        """
        return self._properties['customer_profile_sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def channel_endpoint_type(self):
        """
        :returns: The type of channel endpoint
        :rtype: unicode
        """
        return self._properties['channel_endpoint_type']

    @property
    def channel_endpoint_sid(self):
        """
        :returns: The sid of an channel endpoint
        :rtype: unicode
        """
        return self._properties['channel_endpoint_sid']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Identity resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the CustomerProfilesChannelEndpointAssignmentInstance

        :returns: The fetched CustomerProfilesChannelEndpointAssignmentInstance
        :rtype: twilio.rest.trusthub.v1.customer_profiles.customer_profiles_channel_endpoint_assignment.CustomerProfilesChannelEndpointAssignmentInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the CustomerProfilesChannelEndpointAssignmentInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Trusthub.V1.CustomerProfilesChannelEndpointAssignmentInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class CompositionList(ListResource):

    def __init__(self, version):
        """
        Initialize the CompositionList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.video.v1.composition.CompositionList
        :rtype: twilio.rest.video.v1.composition.CompositionList
        """
        super(CompositionList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Compositions'.format(**self._solution)

    def stream(self, status=values.unset, date_created_after=values.unset,
               date_created_before=values.unset, room_sid=values.unset, limit=None,
               page_size=None):
        """
        Streams CompositionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param CompositionInstance.Status status: Read only Composition resources with this status
        :param datetime date_created_after: Read only Composition resources created on or after this `ISO 8601 <https://en.wikipedia.org/wiki/ISO_8601>`_ date-time with time zone
        :param datetime date_created_before: Read only Composition resources created before this ISO 8601 date-time with time zone
        :param unicode room_sid: Read only Composition resources with this Room SID
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.video.v1.composition.CompositionInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            status=status,
            date_created_after=date_created_after,
            date_created_before=date_created_before,
            room_sid=room_sid,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, status=values.unset, date_created_after=values.unset,
             date_created_before=values.unset, room_sid=values.unset, limit=None,
             page_size=None):
        """
        Lists CompositionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param CompositionInstance.Status status: Read only Composition resources with this status
        :param datetime date_created_after: Read only Composition resources created on or after this `ISO 8601 <https://en.wikipedia.org/wiki/ISO_8601>`_ date-time with time zone
        :param datetime date_created_before: Read only Composition resources created before this ISO 8601 date-time with time zone
        :param unicode room_sid: Read only Composition resources with this Room SID
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.video.v1.composition.CompositionInstance]
        """
        return list(self.stream(
            status=status,
            date_created_after=date_created_after,
            date_created_before=date_created_before,
            room_sid=room_sid,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, status=values.unset, date_created_after=values.unset,
             date_created_before=values.unset, room_sid=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of CompositionInstance records from the API.
        Request is executed immediately

        :param CompositionInstance.Status status: Read only Composition resources with this status
        :param datetime date_created_after: Read only Composition resources created on or after this `ISO 8601 <https://en.wikipedia.org/wiki/ISO_8601>`_ date-time with time zone
        :param datetime date_created_before: Read only Composition resources created before this ISO 8601 date-time with time zone
        :param unicode room_sid: Read only Composition resources with this Room SID
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionPage
        """
        data = values.of({
            'Status': status,
            'DateCreatedAfter': serialize.iso8601_datetime(date_created_after),
            'DateCreatedBefore': serialize.iso8601_datetime(date_created_before),
            'RoomSid': room_sid,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return CompositionPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of CompositionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return CompositionPage(self._version, response, self._solution)

    def create(self, room_sid, video_layout=values.unset,
               audio_sources=values.unset, audio_sources_excluded=values.unset,
               resolution=values.unset, format=values.unset,
               status_callback=values.unset, status_callback_method=values.unset,
               trim=values.unset):
        """
        Create the CompositionInstance

        :param unicode room_sid: The SID of the Group Room with the media tracks to be used as composition sources
        :param dict video_layout: An object that describes the video layout of the composition
        :param list[unicode] audio_sources: An array of track names from the same group room to merge
        :param list[unicode] audio_sources_excluded: An array of track names to exclude
        :param unicode resolution: A string that describes the columns (width) and rows (height) of the generated composed video in pixels
        :param CompositionInstance.Format format: The container format of the composition's media files
        :param unicode status_callback: The URL we should call to send status information to your application
        :param unicode status_callback_method: The HTTP method we should use to call status_callback
        :param bool trim: Whether to clip the intervals where there is no active media in the composition

        :returns: The created CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionInstance
        """
        data = values.of({
            'RoomSid': room_sid,
            'VideoLayout': serialize.object(video_layout),
            'AudioSources': serialize.map(audio_sources, lambda e: e),
            'AudioSourcesExcluded': serialize.map(audio_sources_excluded, lambda e: e),
            'Resolution': resolution,
            'Format': format,
            'StatusCallback': status_callback,
            'StatusCallbackMethod': status_callback_method,
            'Trim': trim,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return CompositionInstance(self._version, payload, )

    def get(self, sid):
        """
        Constructs a CompositionContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.video.v1.composition.CompositionContext
        :rtype: twilio.rest.video.v1.composition.CompositionContext
        """
        return CompositionContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a CompositionContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.video.v1.composition.CompositionContext
        :rtype: twilio.rest.video.v1.composition.CompositionContext
        """
        return CompositionContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Video.V1.CompositionList>'


class CompositionPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the CompositionPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.video.v1.composition.CompositionPage
        :rtype: twilio.rest.video.v1.composition.CompositionPage
        """
        super(CompositionPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of CompositionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.video.v1.composition.CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionInstance
        """
        return CompositionInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Video.V1.CompositionPage>'


class CompositionContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the CompositionContext

        :param Version version: Version that contains the resource
        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.video.v1.composition.CompositionContext
        :rtype: twilio.rest.video.v1.composition.CompositionContext
        """
        super(CompositionContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Compositions/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the CompositionInstance

        :returns: The fetched CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return CompositionInstance(self._version, payload, sid=self._solution['sid'], )

    def delete(self):
        """
        Deletes the CompositionInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Video.V1.CompositionContext {}>'.format(context)


class CompositionInstance(InstanceResource):

    class Status(object):
        ENQUEUED = "enqueued"
        PROCESSING = "processing"
        COMPLETED = "completed"
        DELETED = "deleted"
        FAILED = "failed"

    class Format(object):
        MP4 = "mp4"
        WEBM = "webm"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the CompositionInstance

        :returns: twilio.rest.video.v1.composition.CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionInstance
        """
        super(CompositionInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'status': payload.get('status'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_completed': deserialize.iso8601_datetime(payload.get('date_completed')),
            'date_deleted': deserialize.iso8601_datetime(payload.get('date_deleted')),
            'sid': payload.get('sid'),
            'room_sid': payload.get('room_sid'),
            'audio_sources': payload.get('audio_sources'),
            'audio_sources_excluded': payload.get('audio_sources_excluded'),
            'video_layout': payload.get('video_layout'),
            'resolution': payload.get('resolution'),
            'trim': payload.get('trim'),
            'format': payload.get('format'),
            'bitrate': deserialize.integer(payload.get('bitrate')),
            'size': deserialize.integer(payload.get('size')),
            'duration': deserialize.integer(payload.get('duration')),
            'media_external_location': payload.get('media_external_location'),
            'status_callback': payload.get('status_callback'),
            'status_callback_method': payload.get('status_callback_method'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: CompositionContext for this CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionContext
        """
        if self._context is None:
            self._context = CompositionContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def status(self):
        """
        :returns: The status of the composition
        :rtype: CompositionInstance.Status
        """
        return self._properties['status']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_completed(self):
        """
        :returns: Date when the media processing task finished
        :rtype: datetime
        """
        return self._properties['date_completed']

    @property
    def date_deleted(self):
        """
        :returns: The ISO 8601 date and time in GMT when the composition generated media was deleted
        :rtype: datetime
        """
        return self._properties['date_deleted']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def room_sid(self):
        """
        :returns: The SID of the Group Room that generated the audio and video tracks used in the composition
        :rtype: unicode
        """
        return self._properties['room_sid']

    @property
    def audio_sources(self):
        """
        :returns: The array of track names to include in the composition
        :rtype: list[unicode]
        """
        return self._properties['audio_sources']

    @property
    def audio_sources_excluded(self):
        """
        :returns: The array of track names to exclude from the composition
        :rtype: list[unicode]
        """
        return self._properties['audio_sources_excluded']

    @property
    def video_layout(self):
        """
        :returns: An object that describes the video layout of the composition
        :rtype: dict
        """
        return self._properties['video_layout']

    @property
    def resolution(self):
        """
        :returns: The dimensions of the video image in pixels expressed as columns (width) and rows (height)
        :rtype: unicode
        """
        return self._properties['resolution']

    @property
    def trim(self):
        """
        :returns: Whether to remove intervals with no media
        :rtype: bool
        """
        return self._properties['trim']

    @property
    def format(self):
        """
        :returns: The container format of the composition's media files as specified in the POST request that created the Composition resource
        :rtype: CompositionInstance.Format
        """
        return self._properties['format']

    @property
    def bitrate(self):
        """
        :returns: The average bit rate of the composition's media
        :rtype: unicode
        """
        return self._properties['bitrate']

    @property
    def size(self):
        """
        :returns: The size of the composed media file in bytes
        :rtype: unicode
        """
        return self._properties['size']

    @property
    def duration(self):
        """
        :returns: The duration of the composition's media file in seconds
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def media_external_location(self):
        """
        :returns: The URL of the media file associated with the composition when stored externally
        :rtype: unicode
        """
        return self._properties['media_external_location']

    @property
    def status_callback(self):
        """
        :returns: The URL called to send status information on every composition event.
        :rtype: unicode
        """
        return self._properties['status_callback']

    @property
    def status_callback_method(self):
        """
        :returns: The HTTP method used to call `status_callback`
        :rtype: unicode
        """
        return self._properties['status_callback_method']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The URL of the media file associated with the composition
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the CompositionInstance

        :returns: The fetched CompositionInstance
        :rtype: twilio.rest.video.v1.composition.CompositionInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the CompositionInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Video.V1.CompositionInstance {}>'.format(context)
